@@ -28,6 +28,8 @@ from . types import (
     AssignConstantConfig,
     AssignFormatConfig,
     AssignIdConfig,
+    AssignLengthConfig,
+    CastConfig,
     FilterConfig,
     GlobalStatus,
     JoinConfig,
@@ -140,6 +142,37 @@ def setup_actions_with_args(
                     reverse = reverse,
                 ))
                 continue
+            if action_name == 'assign-length':
+                config.actions.append(AssignLengthConfig(
+                    target = target,
+                    source = source,
+                ))
+                continue
+            if action_name == 'cast':
+                required = options.get('required', False)
+                as_type = options.get('as', 'literal')
+                if as_type in ['boolean']:
+                    as_type = 'bool'
+                if as_type not in ['bool', 'int', 'float', 'str']:
+                    raise ValueError(
+                        f'Unsupported as type: {as_type}'
+                    )
+                assign_default = False
+                default_value = None
+                if 'default' in options:
+                    assign_default = True
+                    default_value = options['default']
+                    if default_value in ['None', 'none', 'Null', 'null']:
+                        default_value = None
+                config.actions.append(CastConfig(
+                    target = target,
+                    source = source,
+                    as_type = as_type,
+                    required = required,
+                    assign_default = assign_default,
+                    default_value = default_value,
+                ))
+                continue
             if action_name == 'filter-empty':
                 config.actions.append(FilterConfig(
                     field = target,
@@ -170,15 +203,26 @@ def setup_actions_with_args(
             if action_name == 'parse':
                 as_type = options.get('as', 'literal')
                 required = options.get('required', False)
-                if as_type not in ['json', 'literal']:
+                if as_type in ['boolean']:
+                    as_type = 'bool'
+                if as_type not in ['bool', 'json', 'literal']:
                     raise ValueError(
                         f'Unsupported as type: {as_type}'
                     )
+                assign_default = False
+                default_value = None
+                if 'default' in options:
+                    assign_default = True
+                    default_value = options['default']
+                    if default_value in ['None', 'none', 'Null', 'null']:
+                        default_value = None
                 config.actions.append(ParseConfig(
                     target = target,
                     source = source,
                     as_type = as_type,
                     required = required,
+                    assign_default = assign_default,
+                    default_value = default_value,
                 ))
                 continue
             if action_name == 'parse-json':
@@ -303,6 +347,10 @@ def do_action(
         return assign_format(row, action)
     if isinstance(action, AssignIdConfig):
         return assign_id(status.id_context_map, row, action)
+    if isinstance(action, AssignLengthConfig):
+        return assign_length(row, action)
+    if isinstance(action, CastConfig):
+        return cast(row, action)
     if isinstance(action, FilterConfig):
         if filter_row(row, action):
             return row
@@ -438,9 +486,9 @@ def search_with_operator(
     row: Row,
     source: str,
 ):
-    or_operator = '\|\|'
-    null_or_operator = '\?\?'
-    operator_group = f'{or_operator}|{null_or_operator}'
+    or_operator = '||'
+    null_or_operator = '??'
+    operator_group = f'{re.escape(or_operator)}|{re.escape(null_or_operator)}'
     matched = re.split(f'({operator_group})', source, 1)
     #ic(source, matched)
     if len(matched) == 1:
@@ -448,10 +496,10 @@ def search_with_operator(
     matched = map(str.strip, matched)
     left, operator, rest = matched
     value, found = search_column_value(row.nested, left)
-    if operator == '||':
+    if operator == or_operator:
         if bool(value):
             return value, found
-    if operator == '??':
+    if operator == null_or_operator:
         if found and value is not None:
             return value, found
     return search_with_operator(row, rest)
@@ -590,27 +638,48 @@ def parse(
                 f'Required field not found, field: {config.source}'
             )
     if found:
-        if type(value) == str:
-            if config.as_type == 'literal':
-                try:
+        if config.as_type == 'literal':
+            try:
+                if type(value) is str:
                     parsed = ast.literal_eval(value)
-                except:
-                    raise ValueError(
-                        f'Failed to parse literal: {value}'
-                    )
-            elif config.as_type == 'json':
-                try:
+                else:
+                    parsed = value
+            except:
+                raise ValueError(
+                    f'Failed to parse literal: {value}'
+                )
+        elif config.as_type == 'json':
+            try:
+                if type(value) is str:
                     parsed = json.loads(value)
-                except:
+                else:
+                    parsed = value
+            except:
+                raise ValueError(
+                    f'Failed to parse JSON: {value}'
+                )
+        elif config.as_type == 'bool':
+            if config.assign_default and value in [None, '']:
+                value = config.default_value
+            if type(value) is bool:
+                parsed = value
+            elif type(value) is str:
+                if value.lower() in ['true', 'yes', 'on', '1']:
+                    parsed = True
+                elif value.lower() in ['false', 'no', 'off', '0']:
+                    parsed = False
+                else:
                     raise ValueError(
-                        f'Failed to parse JSON: {value}'
+                        f'Failed to parse bool: {value}'
                     )
             else:
                 raise ValueError(
-                    f'Unsupported as type: {config.as_type}'
+                    f'Failed to parse bool: {value}'
                 )
         else:
-            parsed = value
+            raise ValueError(
+                f'Unsupported as type: {config.as_type}'
+            )
         set_row_staging_value(row, config.target, parsed)
     return row
 
@@ -635,3 +704,46 @@ def push_field(
             set_row_staging_value(row, config.target, array)
         array.append(source_value)
     return row
+
+def assign_length(
+    row: Row,
+    config: AssignLengthConfig,
+):
+    value, found = search_column_value(row.nested, config.source)
+    if found:
+        set_row_staging_value(row, config.target, len(value))
+    return row
+
+def cast(
+    row: Row,
+    config: CastConfig,
+):
+    value, found = search_column_value(row.nested, config.source)
+    if config.required:
+        if not found:
+            raise ValueError(
+                f'Required field not found, field: {config.source}'
+            )
+    if config.as_type == 'bool':
+        cast_func = bool
+    elif config.as_type == 'int':
+        cast_func = int
+    elif config.as_type == 'float':
+        cast_func = float
+    elif config.as_type == 'str':
+        cast_func = str
+    else:
+        raise ValueError(
+            f'Unsupported as type: {config.as_type}'
+        )
+    try:
+        casted = cast_func(value)
+    except:
+        if config.assign_default:
+            casted = config.default_value
+        else:
+            raise ValueError(
+                f'Failed to cast: {value}'
+            )
+    set_row_staging_value(row, config.target, casted)
+    return row
